
/*%MACRO calc_residual_stats_dp;  */
%MACRO calc_residual_stats_genmod_dp; 

/* Wrapper for calculating residuals at a DP site for Genmod regression . It can only be used in the context of distributed regression macro */

%LOCAL min_count_per_grp_mod allowed_n_grp pct_incr_for_histgrm  resid_vars max_allowed_n_grp;

          DATA &dplocal_lib..&prefix0_dp.resid_detail;
				SET  &prefix_dp.reg_vars(RENAME=(&dependent_vars.=_RESP_for_SSCP_calc_));
			   &dependent_vars.=_RESP_;
			RUN;

			  /* We do not use weight here because it was already included in the difinitions of 
			            _LL_ _PRESIDUAL_  _PRESIDUAL_SQ_ _DRESIDUAL_ _DRESIDUAL_SQ_
			         */
	 
	
				DATA &prefix0_dp.allowed_N_grp;
		         SET  &msoc_lib..&prefix.resid_sum(KEEP=_FREQ_);
			
			    min_count_per_grp_mod=MAX(ROUND(_FREQ_/&max_numb_of_grp.), &min_count_per_grp.,1);	
			/*	 min_count_per_grp_mod=MAX(&min_count_per_grp.,1);	  */
				 max_allowed_n_grp=MAX(INT(_FREQ_/min_count_per_grp_mod),1);
				 allowed_n_grp=MIN(max_allowed_n_grp,&groups.);
				 pct_incr_for_histgrm=MAX(ROUND(100/max_allowed_n_grp,1),1);

				CALL SYMPUT("min_count_per_grp_mod",STRIP(PUT(min_count_per_grp_mod,8.)));
				CALL SYMPUT("allowed_n_grp",STRIP(PUT(allowed_n_grp,8.)));
				CALL SYMPUT("max_allowed_n_grp",STRIP(PUT(max_allowed_n_grp,8.)));
				CALL SYMPUT("pct_incr_for_histgrm",STRIP(PUT(pct_incr_for_histgrm,8.)));
		   RUN;

		   %PUT min_count_per_grp=&min_count_per_grp. ;
		   %PUT min_count_per_grp_mod=&min_count_per_grp_mod. ;
		   %PUT allowed_n_grp=&allowed_n_grp.;
		   %PUT max_allowed_n_grp=&max_allowed_n_grp.;
	
/* The output dataset &prefix0_dp.resid_sum_by_pct has data groupd data by percentiles of
	   predicted outcome (_MEAN_). The number of groups is determined by &min_count_per_grp_mod.
	   and 	&allowed_n_grp. For each group (bin) the macro %sum_by_pct2 calculates
		mean values for each var in &resid_vars  and mean value of predicted outcome ;     
     */	 

		   %LET resid_vars=_RESP_  _RESIDUAL_  _PRESIDUAL_ _RESIDUAL_SQ_  _PRESIDUAL_SQ_  _ADJ_VARIANCE_;

		%sum_by_pct2(tbl_name_in=&dplocal_lib..&prefix0_dp.resid_detail
			                    ,tbl_name_out= &prefix0_dp.resid_sum_by_pct
								,freq=_FREQ_
			                    ,x_var= _MEAN_
                                ,y_var=&resid_vars.
								,stats_for_y_vars=%STR(MEAN= STD= )
								,min_count_per_grp=&min_count_per_grp_mod.
                                ,groups=&allowed_n_grp.);
	 		
		DATA  &msoc_lib..&prefix0.resid_sum_by_pct;
		 LENGTH dp_cd 8;
		  SET &prefix0_dp.resid_sum_by_pct;
			 ARRAY num{*} _NUMERIC_;
		     dp_cd=&dp_cd.;
			    /* In case DP has a a very small dataset. Should almost never happen*/
			  IF &allowed_n_grp. =1 AND _FREQ_ < &min_count_per_grp_mod. THEN 
                                 DO i =1 TO DIM(num);
			 IF LOWCASE(VNAME(num{i})) NE "dp_cd" THEN num{i}=.;
                         IF _N_=1;	
								 END;
			DROP i;
		RUN;

		
		%sum_by_pct2(tbl_name_in=&dplocal_lib..&prefix0_dp.resid_detail
			                    ,tbl_name_out= &prefix0_dp.resid_sum_by_pct2
								,freq=_FREQ_
			                    ,x_var= _MEAN_
                                ,y_var=&resid_vars.
								,stats_for_y_vars=%STR(MEAN= STD= )
								,min_count_per_grp=&min_count_per_grp_mod.
                                ,groups=&max_allowed_n_grp.);
	 		
		DATA  &msoc_lib..&prefix0.resid_sum_by_pct2;
		 LENGTH dp_cd 8;
		  SET &prefix0_dp.resid_sum_by_pct2;
			 ARRAY num{*} _NUMERIC_;
		     dp_cd=&dp_cd.;
			    /* In case DP has a a very small dataset. Should almost never happen*/
			  IF &max_allowed_n_grp. =1 AND _FREQ_ < &min_count_per_grp_mod. THEN 
                                 DO i =1 TO DIM(num);
			 IF LOWCASE(VNAME(num{i})) NE "dp_cd" THEN num{i}=.;
                         IF _N_=1;	
								 END;
			DROP i;
		RUN;

	/* Calculate DP's contribution to the matrix that will be used for calculation of 
		robust sandwich estimator of Cov matrix */

		    %calculate_sscp(tbl_name_in=&prefix_dp.reg_vars
                         ,independent_vars=&independent_vars.
						 ,dependent_vars=
						 ,freq=_FREQ_
                         ,weight=_WEIGHT_for_I1_MATR_
						 ,tbl_name_out=&msoc_lib..&prefix0.sw_matrix);

   %PUT  ------ End of macro calc_residual_stats_dp;
%MEND;