                  
 /*HEADER------------------------------------------------------------------------
|
| PROGRAM:  distributed_regression.sas (macro)
||
| PURPOSE:  The main macro for the distributed regression at the central site
                        Allows to run linear, logistic and Cox regression. Can be extended to run other generalized
                       linear regressions like Poisson and Gamma. 
|
|
| MACRO PARAMETERS:
|
	 INPUT:    
          &reg_ds_in             The name of the input analytic dataset. Must have the same name at all DP sites.

|	     &dp_cd_list              The list data partners (DP) sites separated by space. The mapping of these codes
                                           for Sentinel DPs is defined in the table: map_dpid_siteid_to_dp_cd.
										 	Example, dp_cd_list=1 2 3

	 &regr_type_cd				Defines the type of the regression.	
                                          1- linear; 2- logistic; 10- Cox; 3-Reserved for Poisson

     &dependent_vars           Name of the dependent variable in the regression 

     &independent_vars	 	   List of the independent variables in the regression

	 &censoring_var				 Name of the censoring variable in the Cox regression.
                                            Not relevant for other types of regression. 

	 &censoring_lev				Value of the censoring variable indicating cencored observations.
										  Default 0.

    &ties                              	Specifies the method of handling ties in failure times in Cox regression. 
                                           Allowed values are: BRESLOW or ERFRON 

	 &NOINT						   When set to NOINT the regression is fit without intercept.
      									   Default is blank which fits the model with the intercept.
										    Not relevant for Cox regression

	 &freq								Optional. Names the variable that indicates frequency of an observation

	  &weight						 Optional. Names the variable that indicates weight of an observation.

	 &tbl_intial_est			   Optional. Names the table with initial guess of the regression parameter
										  estimates. Must have a column for each of the parameter estimates which is
                                          named as a corresponding covariate(Same structure as special SAS 
                                         dataset of the TYPE=PARM).  If not specified all initial value of set 
                                         are set to 0. Not relevant for linear regression.

   &max_iter_nb					Maximum number of allowed iterations. Default 10 for testing.
										  Change to 20 in prod.

	&xconv						    Relative convergence criteria. The same definition as the one used by SAS.
										 Default 1e-4


   &tbl_events_time_set      Optional, only relevant for Cox regression. Names the table with all
										   possible value of event times at each DP(no censoring times). Must have
										   one column with the same name as dependent variable. If not specified
										   then correct event time set is created at central site after the data from all DPs 
 										   comes back at the first iteration.

	&alpha					        Sets significance value confidence limits.
										  Default 0.05.

	 &groups					  Number of groups use in the calculation of residuals summary stats.
										Also used for HosmerLemeshow test for Logistic regression.

	wait_time_min			   Minimum time interval for checking of the trigger file: files_done.ok
										Measured in sec. Used by the macro file_watcher. Default 3.

	 wait_time_max			   Maximum time interval for checking of the trigger file: files_done.ok
									 	 Measured in sec. Used by the macro file_watcher. Default for testing 30.
										 Should be changed to a larger value for production.

	 last_runid_in				Optional. Default  is 1. If one wants to run more that one regression
										within the same Sentinel request one should specify last_runid_in=0
										for the first regression calls of this macro and to 1 for the last call.

	  test_env_cd			   Set to 1 to run in the special development/testing enviroment. The directory
									   structure in this enviroment is the same that the one at the central site.
									  When set 1 the program can be executed within a single SAS session
                                       with the code for different DPs running sequentialy.

min_count_per_grp_glob   Sets minimum count per cell for summarized data returned to 
                                          central site. It is only used if a DP site does not specify parameter
                                         min_count_per_grp in their master program for distributed regresion. 
 									     This affects datasets used for residual analysis and some 
                                        goodness of fit measures (ROC and HL statistic for logistic regression). 
										Default value is min_count_per_grp_glob=6.

max_numb_of_grp			   Sets  upper limit to the number groups for summarized data returned to 
                                          the central site. The datasets are used for approximate calculation of
                                          ROC and HL statistic for logistic regression. Normally the number of groups is determined by parameters 
                                          min_count_per_grp or min_count_per_grp_glob. However, for large datasets 
                                          this can result in large amount of data transfered from DPs to the central site.
                                          This is often unnecessary and the parameter put a cap on the number of 
                                           rows returned to the central site.

which Its purpose is to reduce the amount of data brought back to 
                                          to the central site and it  for performace  If parameter 
USAGE EXAMPLES:
                                    Linear:

				   %LET RunID=dr1;
					%LET reg_ds_in=DATA_IN.LINEAR_KARR_2005;
					%LET  dependent_vars=medv;
					%LET independent_vars=crim indus dis dummy_dp_var2 dummy_dp_var3;

					%PUT  RunID=&RunID.;

	 %distributed_regression(RunID=&RunID.
											   ,reg_ds_in=&reg_ds_in. 
											   ,dependent_vars=&dependent_vars.
											   ,independent_vars=&independent_vars.
											   ,dp_cd_list=1 2 3
											   ,NOINT=
											   ,freq=FREQ1
											   ,weight=
											   ,regr_type_cd=1
											   ,alpha=0.05
											   ,groups=10
											   ,wait_time_min=1
	    									   ,wait_time_max=30
											   ,test_env_cd=1
											  ,last_runid_in =1
												) 
                                  ;
 
		             	 * Logistic:

	                %LET RunID=DL16;
					%LET tbl_intial_est=Model_Coeff_4 ;
			 
					%LET reg_ds_in=LINEAR_KARR_2005;
					%LET  dependent_vars=medv_high_flag;
					%LET independent_vars=crim indus dis dummy_dp_var2 dummy_dp_var3;

					%PUT  RunID=&RunID.;

	 					%distributed_regression(RunID=&RunID.
											   ,reg_ds_in=&reg_ds_in. 
											   ,dependent_vars=&dependent_vars.
											   ,independent_vars=&independent_vars.
											   ,dp_cd_list=1 2 3
											   ,NOINT=
											   ,freq=FREQ1
											   ,weight=
											   ,regr_type_cd=2
											   ,tbl_intial_est=&tbl_intial_est.
											   ,alpha=0.05
											  ,XCONV=1e-4
											   ,groups=10
											   ,wait_time_min=1
	    									   ,wait_time_max=30
											   ,max_iter_nb=8
											   ,test_env_cd=1
											  ,last_runid_in =1
											  ,min_count_per_grp_glob=6
												) 
                                  ;

			  		%LET RunID=dc1;
					%LET reg_ds_in=RECID_DR;
					%LET dependent_vars=week;
					%LET independent_vars=fin age  prio;
					%LET censoring_var=arrest;
					%LET censoring_lev=0;
				    %LET tbl_intial_est=Cox_Model_Coeff_3  ;
					%LET tbl_events_time_set=recid_events_time_set;
					  %LET xconv=1e-4;
					     %LET freq=;
						 %LET weight=;

					%PUT  RunID=&RunID.;

					%distributed_regression(RunID=&RunID.
					                                           ,dp_cd_list=1 2 3
															   ,reg_ds_in=&reg_ds_in. 
															   ,dependent_vars=&dependent_vars.
															   ,independent_vars=&independent_vars.
															   ,censoring_var=&censoring_var.
															   ,censoring_lev=0
															   ,NOINT=
															   ,freq=&freq.
															   ,weight=&weight.
															   ,regr_type_cd=10
															   ,tbl_intial_est=&tbl_intial_est.
															   ,tbl_events_time_set=&tbl_events_time_set.
															   ,alpha=0.05
															   ,groups=10
															   ,wait_time_min=3
					    									  ,wait_time_max=30
															  ,last_runid_in =1
															   ,test_env_cd=1
															  ,max_iter_nb=10
															  ,XCONV=&xconv.
																) ;
 
	First version:  02/02/2017, Yury Vilk 


+------------------------------------------------------------------------HEADER*/
 %MACRO distributed_regression(RunID=
                                                       ,dp_cd_list=1 2 3
													   ,reg_ds_in= 
													   ,dependent_vars=
													   ,independent_vars=
													   ,censoring_var=
													   ,censoring_lev=0
													   ,strata_vars=
													   ,ties=BRESLOW
													   ,NOINT=
													   ,freq=
													   ,weight=
													   ,regr_type_cd=1
													   ,tbl_intial_est=
													   ,tbl_events_time_set=
													   ,alpha=0.05
													   ,groups=10
													   ,wait_time_min=3
			    									  ,wait_time_max=30
													  ,last_runid_in =1
													   ,test_env_cd=0
													  ,max_iter_nb=10
													  ,XCONV=1e-4
													  ,max_numb_of_grp=10000
													   ,min_count_per_grp_glob=6
														) ;

 %LOCAL    iter_nb  reg_conv_in  reg_conv_msg
    prev_reg_conv_in reg_conv_step_in iter_nb_at_conv numb_iter_after_conv last_iter_in end_job_dp_in
   track_tbl_csv_for_last_iter prefix0 prefix  prefix_prev Intercept 
    rc_msoc rc_dplocal rc_infolder rc_sasmacr
	exist_init_events_time_set count_independent_vars
	addn_cox_vars_for_dp
     infolder_lib msoc_lib dplocal_lib prefix0_dp prefix_dp 
   ;
	   	 

   %GLOBAL m_return_cd    ;
  %LET m_return_cd=0;

   %LET RunID=%lowcase(&RunID.);
    %LET tbl_events_time_set=%lowcase(&tbl_events_time_set.);

   %PUT;
   %PUT -----   Starting distributed regression with following parameters: ;
   %PUT;


%PUT RunID=&RunID. ;
%PUT reg_ds_in=&reg_ds_in.; 
%PUT dependent_vars=&dependent_vars.;
%PUT independent_vars=&independent_vars.;
%PUT censoring_var=&censoring_var. ;
%PUT strata_vars=&strata_vars.;
%PUT NOINT=&NOINT.;
%PUT freq=&freq.;
%PUT weight=&weight.;
%PUT regr_type_cd=&regr_type_cd.;
%PUT tbl_intial_est=&tbl_intial_est.;
%PUT alpha=&alpha.;
%PUT groups=&groups.;
%PUT wait_time_min=&wait_time_min.;
%PUT wait_time_max=&wait_time_max.;
%PUT last_runid_in =&last_runid_in.;
%PUT test_env_cd=&test_env_cd.;
%PUT dp_cd_list=&dp_cd_list.;
%PUT max_iter_nb=&max_iter_nb.;
%PUT XCONV=&XCONV.;
%PUT max_numb_of_grp=&max_numb_of_grp.;
%PUT min_count_per_grp_glob=&min_count_per_grp_glob.;
 
 
			 /* The parameter test_env_cd affects a number of parameters in the DP part of the program:
      
   /*test_env_cd=0: uses production like directory structure with full separate directory structure for each DP.
	   In this case DP specific suffixes and prefixes are 
      not necessary because the directories are not shared.
      	For example, prefix0_dp=&RunID._;  msoc_lib=msoc;

	 test_env_cd=1 uses central site directory structure. It affects a number of parameters in the DP part of the program.
      This makes dataset within the same directory unique for a dp_cd. 
        For example, prefix0_dp=&RunID._&dp_cd._ ; msoc_lib=msoc&dp_cd.; reg_ds_dp_in=&reg_ds_in._&dp_cd.
	 %LET  prefix_dp=&prefix.dp&dp_cd._ ; 	   
   */

		  	/* Reset SAS automatic error variable.*/
	%IF &test_env_cd. NE 0 %THEN %LET SYSCC=0; 

	 %LET runid_return_cd=0;
	 %LET runid_return_msg=;


		   %LET dp_cd_list=%SYSFUNC(COMPBL(&dp_cd_list.));
	       %LET independent_vars=%SYSFUNC(COMPBL(&independent_vars.));

	      %LET count_independent_vars=%sysfunc(countw(&independent_vars.));
		  %PUT  count_independent_vars=&count_independent_vars.;

		 	%delete_ext_file(file_name=&infolder.job_started_central.ok, warning_flag=0);	 

		      	/* Do some initial error handling */

		%IF    %LENGTH(&reg_ds_in.)=0	
            OR  %LENGTH(&dp_cd_list.)=0 
			OR %LENGTH(&dependent_vars.)=0  			
			OR %LENGTH(&independent_vars.)=0 
			OR (%LENGTH(&censoring_var.)=0 AND &regr_type_cd. =10 )
      %THEN 
							%DO;
     %LET runid_return_cd=2111;
	%LET runid_return_msg=%SYSFUNC(COMPBL(%STR(Error:  Main program at the central site will stop executing. 
                              One of the requiered parameters is not specified: 
							   reg_ds_in=&reg_ds_in. dp_cd_list=&dp_cd_list. dependent_vars=&dependent_vars. 
                               independent_vars=&independent_vars. 
                               [regr_type_cd=&regr_type_cd. AND censoring_var=&censoring_var.]	
			                                                       )));	
				%GOTO endmac_fatal_err;
			  				%END;

			   %LET rc_msoc=%SYSFUNC(FILEEXIST("&msoc."));
			   %LET rc_dplocal=%SYSFUNC(FILEEXIST("&dplocal."));
			   %LET rc_infolder=%SYSFUNC(FILEEXIST("&infolder."));
			   %LET rc_sasmacr=%SYSFUNC(FILEEXIST("&sasmacr."));

		%IF    &SYSCC. > 4	
            OR &rc_msoc. NE 1 
			OR &rc_dplocal. NE 1 			
			OR &rc_infolder. NE 1
			OR &rc_sasmacr. NE 1
      %THEN 
							%DO;
     %LET runid_return_cd=2110;
	%LET runid_return_msg=%SYSFUNC(COMPBL(%STR(Error:  Main program at the central site will stop executing. 
                              The most likely source of errors are incorrect references to: folders, filnames or SAS libraries: 
							   rc_msoc=&rc_msoc. rc_dploca=&rc_dplocal. rc_infolder=&rc_infolder. rc_sasmacr=&rc_sasmacr.	
			                                                       )));	
				%GOTO endmac_fatal_err;
			  				%END;		

		     %IF &runid_return_cd.=0 %THEN 
					%DO;
					  %create_empty_file(file_nm=&infolder.job_started_central.ok); 
	%IF &m_return_cd. >0 %THEN  
										%DO;
%LET runid_return_cd=2109;
%LET runid_return_msg=Error:  Main program at the central site will stop executing. Can not write to the folder &infolder..;
						%GOTO endmac_fatal_err;
					  				    %END;
					   %END;

	 /* Initialize variables before loop */
                          		/* dp_cd=0 is set for central site */

   %LET dp_cd=0;
   %LET prefix0=&RunID._;
   %LET  prefix0_dp=&prefix0.;

   %LET iter_nb=1;	
  %LET  reg_conv_in=0; 
   %LET reg_conv_msg=;
   %LET reg_conv_step_in=0;
   %LET prev_reg_conv_in=0;
    %LET numb_iter_after_conv=1;
   %LET iter_nb_at_conv=%eval(&max_iter_nb. +1);
   %LET last_iter_in=0;
   %LET  end_job_dp_in=0;
  
  %LET Intercept=Intercept;

    %LET infolder_lib=infolder;
	%LET msoc_lib=msoc;
   	%LET dplocal_lib=dplocal;

	     /*Set some additional macro variables for Cox regression */

  %IF &regr_type_cd. =10 %THEN  
							  %DO;
                 %LET NOINT=NOINT;
                
		%LOCAL exist_init_events_time_set class_strata_vars  by_strata_vars pos_dp_cd_in_strata_vars ties_sh;

				 %LET class_strata_vars=;
	             %LET by_strata_vars=;
				 %LET pos_dp_cd_in_strata_vars=0;
				  %LET last_strata_var=;
				  %LET fst_by_strata_var=_N_;
				  %LET addn_cox_vars_for_dp=%SYSFUNC(COMPBL(censoring_var  censoring_lev strata_vars 
                                                                                           tbl_events_time_set exist_init_events_time_set 
                                                                                           class_strata_vars  by_strata_vars pos_dp_cd_in_strata_vars 
                                                                                           last_strata_var fst_by_strata_var ties ties_sh));

			%IF  %LENGTH(&strata_vars.) >0  %THEN 
	                                  %DO;
				 %LET strata_vars= %SYSFUNC(COMPBL(&strata_vars.));
                 %LET by_strata_vars=BY &strata_vars.;
				 %LET class_strata_vars=CLASS &strata_vars.;
				  %LET pos_dp_cd_in_strata_vars=%SYSFUNC(INDEXW(&strata_vars., dp_cd));
				  %LET last_strata_var=%SCAN(&strata_vars.,-1, %STR( ))	; 
				  %LET fst_by_strata_var=FIRST.&last_strata_var.;

		  %IF &pos_dp_cd_in_strata_vars. >0 %THEN 
		                                                            %DO;
                                                      %LET tbl_events_time_set=;
			%PUT Ignore tbl_events_time_set and set it to blank because dp_cd is part of strata_vars.;
													                %END;					
										%END;  /* End of do-group %LENGTH(&strata_vars.) >0 */

				%IF &pos_dp_cd_in_strata_vars. =0 %THEN  %LET numb_iter_after_conv=2;

					%LET exist_init_events_time_set=0;
	         %isdata(dataset=infolder.&tbl_events_time_set.);

	       %IF &nobs.  NE 0 %THEN %LET exist_init_events_time_set=1;

			 %LET ties=%LOWCASE(&ties.);
	        %LET ties_sh=%SUBSTR(&ties.,1,1);

			%PUT ties=&ties;
            %PUT ties_sh=&ties_sh.;
			%PUT pos_dp_cd_in_strata_vars=&pos_dp_cd_in_strata_vars.;
			%PUT tbl_events_time_set=&tbl_events_time_set;
			%PUT exist_init_events_time_set=&exist_init_events_time_set.;

				             %END; /* End of do-group &regr_type_cd. =10 */ 
	%ELSE 			 %DO;
	                %LET censoring_var=;
					%LET numb_iter_after_conv=1;
					%LET addn_cox_vars_for_dp=;
					         %END;

  %IF %LENGTH(&NOINT.) NE 0 %THEN %LET Intercept=;	

	/* Initialize to blank. Will be set to correct value only when the last_iter_in=*/
%LET track_tbl_csv_for_last_iter= ;
 
       		 %PUT numb_iter_after_conv=&numb_iter_after_conv.;
			%PUT addn_cox_vars_for_dp=&addn_cox_vars_for_dp.;
						   
 	  %DO %UNTIL(&last_iter_in.=1 );

%LET prefix=&prefix0.i&iter_nb._;
%LET  prefix_dp=&prefix. ;
%LET prefix_prev=&prefix0.i%eval(&iter_nb.-1)_  ;

%PUT prefix=&prefix.;
%PUT prefix_prev=&prefix_prev.;

		   /* Prepare data to be sent to DPs */

    %d_reg_central_step1;

	       /*Execute distributed regression code at each data partner. Test enviroments only */

		%IF  &test_env_cd. NE 0 %THEN
		                   			%DO;
	%_t_loop_through_dp(prefix=&prefix., dp_cd_list=&dp_cd_list.);
									 %END;
				          /* Central Step 2*/

%LET prev_reg_conv_in=&reg_conv_in.;

    /* Receive data from DPs and calculate various statistics. Determine if further iterations are necessary*/

  %d_reg_central_step2 ;

	  	%LET iter_nb=%eval(&iter_nb. +1);

  %END; /* End of DO UNTIL(&last_iter_in.=1);	*/

  	 %create_status_time_rpt(rpt_dir_out=&msoc. , prefix_r=&prefix0. )  ;

    %endmac_fatal_err:
	

    %PUT;
	%PUT  RunID=&RunID. ; 
	%PUT syscc=&syscc.;
    %PUT  runid_return_cd=&runid_return_cd. ; 
     %PUT  runid_return_msg=&runid_return_msg. ;
     %PUT -----   End of the macro distributed_regression;
   	 %PUT;

	   %PUT;
	   %PUT -----   End of the macro distributed_regression;
   	   %PUT;

	%MEND;

